﻿using System;
using System.Text;
using System.Threading;

using RabbitMQ.Client;
using RabbitMQ.Client.Events;

namespace TestConsumer
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            ConnectionFactory factory = new ConnectionFactory() { HostName = "localhost" };

            using(IConnection connection = factory.CreateConnection())
            {
                using(IModel model = connection.CreateModel())
                {
                    model.QueueDeclare
                    (
                        queue      : "task_queue",
                        durable    : true,
                        exclusive  : false,
                        autoDelete : false,
                        arguments  : null
                    );

                    model.BasicQos(prefetchSize : 0, prefetchCount : 1, global : false);

                    WriteLog("메시지 대기...");

                    EventingBasicConsumer consumer = new EventingBasicConsumer(model);

                    consumer.Received += (sender, e) =>
                    {
                        byte[] messageArray = e.Body;

                        string message = Encoding.UTF8.GetString(messageArray);

                        WriteLog("수신 : {0}", message);

                        int dotCount = message.Split('.').Length - 1;

                        Thread.Sleep(dotCount * 1000);

                        WriteLog("작업을 완료했습니다.");

                        model.BasicAck
                        (
                            deliveryTag : e.DeliveryTag,
                            multiple    : false
                        );
                    };

                    model.BasicConsume
                    (
                        queue    : "task_queue",
                        autoAck  : false,
                        consumer : consumer
                    );

                    Console.WriteLine("프로그램을 종료하기 위해 아무 키나 눌러 주시기 바랍니다.");

                    Console.ReadKey(true);
                }
            }
        }

        #endregion
        #region 로그 작성하기 - WriteLog(format, parameterArray)

        /// <summary>
        /// 로그 작성하기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private static void WriteLog(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            string log = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);

            Console.WriteLine(log);
        }

        #endregion
    }
}