﻿using System;
using System.Text;

using RabbitMQ.Client;

namespace TestProducer
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        private static void Main(string[] argumentArray)
        {
            ConnectionFactory factory = new ConnectionFactory() { HostName = "localhost" };

            using(IConnection connection = factory.CreateConnection())
            {
                using(IModel model = connection.CreateModel())
                {
                    model.QueueDeclare
                    (
                        queue      : "task_queue",
                        durable    : true,
                        exclusive  : false,
                        autoDelete : false,
                        arguments  : null
                    );

                    string message = GetMessage(argumentArray);

                    byte[] messageArray = Encoding.UTF8.GetBytes(message);

                    IBasicProperties properties = model.CreateBasicProperties();

                    properties.Persistent = true;

                    model.BasicPublish
                    (
                        exchange        : "",
                        routingKey      : "task_queue",
                        basicProperties : properties,
                        body            : messageArray
                    );

                    WriteLog("송신 : {0}", message);
                }
            }

            Console.WriteLine("프로그램을 종료하려면 아무 키나 눌러 주시기 바랍니다.");

            Console.ReadKey(true);
        }

        #endregion

        #region 메시지 구하기 - GetMessage(argumentArray)

        /// <summary>
        /// 메시지 구하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        /// <returns>메시지</returns>
        private static string GetMessage(string[] argumentArray)
        {
            return ((argumentArray.Length > 0) ? string.Join(" ", argumentArray) : "Hello World!");
        }

        #endregion
        #region 로그 작성하기 - WriteLog(format, parameterArray)

        /// <summary>
        /// 로그 작성하기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private static void WriteLog(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            string log = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);

            Console.WriteLine(log);
        }

        #endregion
    }
}